/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package biojava1test;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Iterator;
import org.biojava.bio.Annotation;

import org.biojava.bio.seq.DNATools;
import org.biojava.bio.seq.Feature;
//import org.biojava.bio.seq.Feature.Template;
import org.biojava.bio.seq.FeatureHolder;
import org.biojava.bio.seq.Sequence;
import org.biojava.bio.seq.SequenceIterator;
import org.biojava.bio.seq.io.SeqIOTools;
import org.biojava.bio.seq.io.SymbolTokenization;
import org.biojava.bio.symbol.FiniteAlphabet;
import org.biojava.bio.symbol.RangeLocation;
import org.biojava.bio.symbol.SimpleSymbolList;
import org.biojava.bio.symbol.Symbol;
import org.biojava.bio.symbol.SymbolList;
import org.biojava.bio.symbol.SymbolListViews;

/**
 *
 * @author caste
 */
public class Main {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) throws Exception {
            //e.g. 1: Alphabet generation
        FiniteAlphabet dna=DNATools.getDNA();   // Generate alphabet
        Iterator dnaSymbols = dna.iterator();
        while (dnaSymbols.hasNext()) {          // Print alphabet symbols
            Symbol s = (Symbol) dnaSymbols.next();
            System.out.println(s.getName());
        }
        System.out.println();

            //e.g. 2: SymbolList generation
        String seqString = "GATTACA";
        SymbolTokenization dnaToke;
        SymbolList seq=null;
        String seqString2=null;
        dnaToke = dna.getTokenization("token"); // This object is used to translate a string into a SymbolList using only the dna alphabet symbols (see biojava javadocs 1.8)

        seq = new SimpleSymbolList(dnaToke, seqString);      // Generate SymbolList
        seqString2 = dnaToke.tokenizeSymbolList(seq);
        
        System.out.println("Strings match: " + seqString2.equalsIgnoreCase(seqString));

        System.out.println("Look at my sequence " + seq.seqString());
        System.out.println("Alphabet = " + seq.getAlphabet().getName());
        System.out.println("Length = " + seq.length());
        System.out.println("First symbol = " + seq.symbolAt(1).getName());
        System.out.println();

            // e.g. 3: backward sequence generation
        SymbolList backwardSeq=null;
        backwardSeq = DNATools.reverseComplement(seq);
        
        System.out.println("First base in seq: " + seq.symbolAt(1).getName());
        System.out.println("Last base in backwardSeq: " + backwardSeq.symbolAt(backwardSeq.length()).getName());
        System.out.println("Seq: " + seq.seqString());
        System.out.println("backwardSeq: " + backwardSeq.seqString());
        System.out.println();

            // e.g. 4: load fasta file into Sequence object and compute cg-content
        String fileName = "ColiphagePhi-X174.fasta";

	// Set up sequence iterator
	BufferedReader br = new BufferedReader(new FileReader(fileName));
	SequenceIterator stream = SeqIOTools.readFastaDNA(br);

	// Iterate over all sequences in the stream
        Sequence seq1=null;
	while (stream.hasNext()) {
	    seq1= stream.nextSequence();
	    int gc = 0;
	    for (int pos = 1; pos <= seq1.length(); ++pos) {
		Symbol sym = seq1.symbolAt(pos);
		if (sym == DNATools.g() || sym == DNATools.c())
		    ++gc;
            }
            System.out.println("Sequence: "+seq1.getName() + ", "+seq1.getURN()+ ". gc: "+gc+", gc content:" + ((double)gc / seq1.length()*100) +"%, length: "+seq1.length());
            System.out.println(seq1.seqString().substring(0, 10)+"..."+seq1.seqString().substring(seq1.length()-10, seq1.length()));
	}
        System.out.println();

            // e.g. 5: repeat e.g. 4 by adding symbols RRRRR after the second nucleotide of the fasta file and find with which nucleotides the R symbol matches

        int gc = 0;
        for (int pos = 1; pos <= seq1.length(); ++pos) {
            Symbol sym = seq1.symbolAt(pos);
            if (sym == DNATools.g() || sym == DNATools.c())
                ++gc;
        }
        System.out.println("Sequence: "+seq1.getName() + ", "+seq1.getURN()+ ". gc: "+gc+", gc content:" + ((double)gc / seq1.length()*100) +"%, length: "+seq1.length());
        System.out.println(seq1.seqString().substring(0, 10)+"..."+seq1.seqString().substring(seq1.length()-10, seq1.length()));
        Symbol s=seq1.symbolAt(3);
        FiniteAlphabet a=(FiniteAlphabet)s.getMatches();
        Iterator rSymbols = a.iterator();
        System.out.println("R("+s.getName()+") matches with:");
        while (rSymbols.hasNext()) {          // Print alphabet symbols
            Symbol s1 = (Symbol) rSymbols.next();
            System.out.println(s1.getName());
        }
	System.out.println();

            // e.g. 6: Sequence annotation
        System.out.println("Seq1 annotations:");
        Annotation seq1An = seq1.getAnnotation();
        for (Iterator i = seq1An.keys().iterator(); i.hasNext(); ) {
            Object key = i.next();
            Object value = seq1An.getProperty(key);
            System.out.println(key.toString() + ": " + value.toString());
        }
        System.out.println();

            // e.g. 7: Sequence annotation (features)
        // Generate a feature
        Feature.Template template = new Feature.Template();
        template.type = "TestFeature";
        template.source = "Test";
        template.location = new RangeLocation(100, 200);
        template.annotation = Annotation.EMPTY_ANNOTATION;
        seq1.createFeature(template);

        System.out.println("Features:");
        printFeatures(seq1, "");
        System.out.println();


            // e.g. 8: Load GenBank file
        // Set up sequence iterator
        String fileName2 = "sequence.gb";
	BufferedReader br2 = new BufferedReader(new FileReader(fileName2));
	SequenceIterator stream2 = SeqIOTools.readGenbank(br2);

	// Iterate over all sequences in the stream
        Sequence seq2=null;
	while (stream2.hasNext()) {
	    seq2= stream2.nextSequence();
	    int gc2 = 0;
	    for (int pos = 1; pos <= seq2.length(); ++pos) {
		Symbol sym = seq2.symbolAt(pos);
		if (sym == DNATools.g() || sym == DNATools.c())
		    ++gc2;
            }
            System.out.println("Sequence: "+seq2.getName() + ", "+seq2.getURN()+ ". gc: "+gc+", gc content:" + ((double)gc / seq2.length()*100) +"%, length: "+seq2.length());
            System.out.println(seq2.seqString().substring(0, 10)+"..."+seq2.seqString().substring(seq2.length()-10, seq2.length()));
	}
        System.out.println("Seq2 annotations:");
        Annotation seq2An = seq2.getAnnotation();
        for (Iterator i = seq2An.keys().iterator(); i.hasNext(); ) {
            Object key = i.next();
            Object value = seq2An.getProperty(key);
            System.out.println(key.toString() + ": " + value.toString());
        }
        System.out.println();
        
        System.out.println("Features:");
        printFeatures(seq2, "");
        System.out.println();

        System.out.println(dna.getGapSymbol().getName());

            // e.g. 9: compute sequence translation, protein,... using DNATools
//        Sequence seq3=DNATools.createDNASequence("ATACCATATTGGGG", "My new Sequence");
//        SymbolList symbList3Complement=DNATools.complement(seq3);
//        Sequence seq3Complement=symbList3Complement.

        // e.g. 9: from slide 51: using SymbolListViews

        try { //make a DNA SymbolList
                SymbolList dna1 = DNATools.createDNA("atgcccgcgtaa");
                System.out.println("Length of dna1 " + dna1.length());
                //get a Codon View (window size of three)
                SymbolList codons = SymbolListViews.windowedSymbolList(dna1, 3);
                System.out.println("Length of codons " + codons.length());//get a Triplet View
                System.out.println("Codons:");
                for(int i=0; i<codons.length(); i++){
                    System.out.println(codons.symbolAt(i+1));
                }

                SymbolList triplets = SymbolListViews.orderNSymbolList(dna1, 3);
                System.out.println("Length of triplets1 "+ triplets.length());
                System.out.println("Triplets:");
                for(int i=0; i<triplets.length(); i++){
                    System.out.println(triplets.symbolAt(i+1));
                }
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }

    }

    public static void printFeatures(FeatureHolder fh, String prefix) {
        for (Iterator i = fh.features(); i.hasNext(); ) {
            Feature f = (Feature) i.next();
            System.out.print(prefix);
            System.out.print(f.getType());
            System.out.print(" at ");
            System.out.println(f.getLocation().toString());
            //if(f.features()!=null)
                printFeatures(f, prefix + "    ");
        }
    }
}
